@@ -23,12 +23,13 @@
 
 #include "php_phongo.h"
 #include "phongo_error.h"
+#include "Decimal128_arginfo.h"
 
 zend_class_entry* php_phongo_decimal128_ce;
 
 /* Initialize the object and return whether it was successful. An exception will
  * be thrown on error. */
-static bool php_phongo_decimal128_init(php_phongo_decimal128_t* intern, const char* value) /* {{{ */
+static bool php_phongo_decimal128_init(php_phongo_decimal128_t* intern, const char* value)
 {
 	if (!bson_decimal128_from_string(value, &intern->decimal)) {
 		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Error parsing Decimal128 string: %s", value);
@@ -38,11 +39,11 @@ static bool php_phongo_decimal128_init(php_phongo_decimal128_t* intern, const ch
 	intern->initialized = true;
 
 	return true;
-} /* }}} */
+}
 
 /* Initialize the object from a HashTable and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_decimal128_init_from_hash(php_phongo_decimal128_t* intern, HashTable* props) /* {{{ */
+static bool php_phongo_decimal128_init_from_hash(php_phongo_decimal128_t* intern, HashTable* props)
 {
 	zval* dec;
 
@@ -52,9 +53,9 @@ static bool php_phongo_decimal128_init_from_hash(php_phongo_decimal128_t* intern
 
 	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "%s initialization requires \"dec\" string field", ZSTR_VAL(php_phongo_decimal128_ce->name));
 	return false;
-} /* }}} */
+}
 
-static HashTable* php_phongo_decimal128_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp) /* {{{ */
+static HashTable* php_phongo_decimal128_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp)
 {
 	php_phongo_decimal128_t* intern;
 	HashTable*               props;
@@ -78,44 +79,33 @@ static HashTable* php_phongo_decimal128_get_properties_hash(phongo_compat_object
 	}
 
 	return props;
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\BSON\Decimal128::__construct(string $value)
-   Construct a new BSON Decimal128 type */
-static PHP_METHOD(Decimal128, __construct)
+/* Construct a new BSON Decimal128 type */
+static PHP_METHOD(MongoDB_BSON_Decimal128, __construct)
 {
-	zend_error_handling      error_handling;
 	php_phongo_decimal128_t* intern;
 	char*                    value;
 	size_t                   value_len;
 
 	intern = Z_DECIMAL128_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &value, &value_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_STRING(value, value_len)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_decimal128_init(intern, value);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\BSON\Decimal128 MongoDB\BSON\Decimal128::__set_state(array $properties)
-*/
-static PHP_METHOD(Decimal128, __set_state)
+static PHP_METHOD(MongoDB_BSON_Decimal128, __set_state)
 {
-	zend_error_handling      error_handling;
 	php_phongo_decimal128_t* intern;
 	HashTable*               props;
 	zval*                    array;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(array)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	object_init_ex(return_value, php_phongo_decimal128_ce);
 
@@ -123,57 +113,38 @@ static PHP_METHOD(Decimal128, __set_state)
 	props  = Z_ARRVAL_P(array);
 
 	php_phongo_decimal128_init_from_hash(intern, props);
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\Decimal128::__toString()
-*/
-static PHP_METHOD(Decimal128, __toString)
+static PHP_METHOD(MongoDB_BSON_Decimal128, __toString)
 {
-	zend_error_handling      error_handling;
 	php_phongo_decimal128_t* intern;
 	char                     outbuf[BSON_DECIMAL128_STRING];
 
 	intern = Z_DECIMAL128_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	bson_decimal128_to_string(&intern->decimal, outbuf);
 
 	RETURN_STRING(outbuf);
-} /* }}} */
+}
 
-/* {{{ proto array MongoDB\BSON\Decimal128::jsonSerialize()
-*/
-static PHP_METHOD(Decimal128, jsonSerialize)
+static PHP_METHOD(MongoDB_BSON_Decimal128, jsonSerialize)
 {
-	zend_error_handling      error_handling;
 	php_phongo_decimal128_t* intern;
 	char                     outbuf[BSON_DECIMAL128_STRING] = "";
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	intern = Z_DECIMAL128_OBJ_P(getThis());
 
 	array_init_size(return_value, 1);
 	bson_decimal128_to_string(&intern->decimal, outbuf);
 	ADD_ASSOC_STRING(return_value, "$numberDecimal", outbuf);
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\Decimal128::serialize()
-*/
-static PHP_METHOD(Decimal128, serialize)
+static PHP_METHOD(MongoDB_BSON_Decimal128, serialize)
 {
-	zend_error_handling      error_handling;
 	php_phongo_decimal128_t* intern;
 	zval                     retval;
 	php_serialize_data_t     var_hash;
@@ -182,12 +153,7 @@ static PHP_METHOD(Decimal128, serialize)
 
 	intern = Z_DECIMAL128_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	bson_decimal128_to_string(&intern->decimal, outbuf);
 	array_init_size(&retval, 1);
@@ -202,13 +168,10 @@ static PHP_METHOD(Decimal128, serialize)
 
 	smart_str_free(&buf);
 	zval_ptr_dtor(&retval);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\BSON\Decimal128::unserialize(string $serialized)
-*/
-static PHP_METHOD(Decimal128, unserialize)
+static PHP_METHOD(MongoDB_BSON_Decimal128, unserialize)
 {
-	zend_error_handling      error_handling;
 	php_phongo_decimal128_t* intern;
 	char*                    serialized;
 	size_t                   serialized_len;
@@ -217,12 +180,9 @@ static PHP_METHOD(Decimal128, unserialize)
 
 	intern = Z_DECIMAL128_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_STRING(serialized, serialized_len)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	PHP_VAR_UNSERIALIZE_INIT(var_hash);
 	if (!php_var_unserialize(&props, (const unsigned char**) &serialized, (unsigned char*) serialized + serialized_len, &var_hash)) {
@@ -236,20 +196,16 @@ static PHP_METHOD(Decimal128, unserialize)
 
 	php_phongo_decimal128_init_from_hash(intern, HASH_OF(&props));
 	zval_ptr_dtor(&props);
-} /* }}} */
+}
 
-/* {{{ proto array MongoDB\Driver\Decimal128::__serialize()
-*/
-static PHP_METHOD(Decimal128, __serialize)
+static PHP_METHOD(MongoDB_BSON_Decimal128, __serialize)
 {
 	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETURN_ARR(php_phongo_decimal128_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true));
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Decimal128::__unserialize(array $data)
-*/
-static PHP_METHOD(Decimal128, __unserialize)
+static PHP_METHOD(MongoDB_BSON_Decimal128, __unserialize)
 {
 	zval* data;
 
@@ -258,53 +214,12 @@ static PHP_METHOD(Decimal128, __unserialize)
 	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_decimal128_init_from_hash(Z_DECIMAL128_OBJ_P(getThis()), Z_ARRVAL_P(data));
-} /* }}} */
+}
 
-/* {{{ MongoDB\BSON\Decimal128 function entries */
-/* clang-format off */
-ZEND_BEGIN_ARG_INFO_EX(ai_Decimal128___construct, 0, 0, 1)
-	ZEND_ARG_INFO(0, value)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Decimal128___set_state, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, properties, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(ai_Decimal128___toString, 0, 0, IS_STRING, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Decimal128___unserialize, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, data, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Decimal128_jsonSerialize, 0, 0, IS_ARRAY, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Decimal128_unserialize, 0, 0, 1)
-	ZEND_ARG_INFO(0, serialized)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Decimal128_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_decimal128_me[] = {
-	PHP_ME(Decimal128, __construct, ai_Decimal128___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Decimal128, __serialize, ai_Decimal128_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Decimal128, __set_state, ai_Decimal128___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-	PHP_ME(Decimal128, __toString, ai_Decimal128___toString, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Decimal128, __unserialize, ai_Decimal128___unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Decimal128, jsonSerialize, ai_Decimal128_jsonSerialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Decimal128, serialize, ai_Decimal128_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Decimal128, unserialize, ai_Decimal128_unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-};
-/* clang-format on */
-/* }}} */
-
-/* {{{ MongoDB\BSON\Decimal128 object handlers */
+/* MongoDB\BSON\Decimal128 object handlers */
 static zend_object_handlers php_phongo_handler_decimal128;
 
-static void php_phongo_decimal128_free_object(zend_object* object) /* {{{ */
+static void php_phongo_decimal128_free_object(zend_object* object)
 {
 	php_phongo_decimal128_t* intern = Z_OBJ_DECIMAL128(object);
 
@@ -314,9 +229,9 @@ static void php_phongo_decimal128_free_object(zend_object* object) /* {{{ */
 		zend_hash_destroy(intern->properties);
 		FREE_HASHTABLE(intern->properties);
 	}
-} /* }}} */
+}
 
-static zend_object* php_phongo_decimal128_create_object(zend_class_entry* class_type) /* {{{ */
+static zend_object* php_phongo_decimal128_create_object(zend_class_entry* class_type)
 {
 	php_phongo_decimal128_t* intern = zend_object_alloc(sizeof(php_phongo_decimal128_t), class_type);
 
@@ -326,9 +241,9 @@ static zend_object* php_phongo_decimal128_create_object(zend_class_entry* class_
 	intern->std.handlers = &php_phongo_handler_decimal128;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static zend_object* php_phongo_decimal128_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
+static zend_object* php_phongo_decimal128_clone_object(phongo_compat_object_handler_type* object)
 {
 	php_phongo_decimal128_t* intern;
 	php_phongo_decimal128_t* new_intern;
@@ -345,33 +260,23 @@ static zend_object* php_phongo_decimal128_clone_object(phongo_compat_object_hand
 	new_intern->initialized = true;
 
 	return new_object;
-} /* }}} */
+}
 
-static HashTable* php_phongo_decimal128_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_decimal128_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	*is_temp = 1;
 	return php_phongo_decimal128_get_properties_hash(object, true);
-} /* }}} */
+}
 
-static HashTable* php_phongo_decimal128_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
+static HashTable* php_phongo_decimal128_get_properties(phongo_compat_object_handler_type* object)
 {
 	return php_phongo_decimal128_get_properties_hash(object, false);
-} /* }}} */
-/* }}} */
+}
 
-void php_phongo_decimal128_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_decimal128_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\BSON", "Decimal128", php_phongo_decimal128_me);
-	php_phongo_decimal128_ce                = zend_register_internal_class(&ce);
+	php_phongo_decimal128_ce                = register_class_MongoDB_BSON_Decimal128(php_phongo_decimal128_interface_ce, php_phongo_json_serializable_ce, php_phongo_type_ce, zend_ce_serializable);
 	php_phongo_decimal128_ce->create_object = php_phongo_decimal128_create_object;
-	PHONGO_CE_FINAL(php_phongo_decimal128_ce);
-
-	zend_class_implements(php_phongo_decimal128_ce, 1, php_phongo_decimal128_interface_ce);
-	zend_class_implements(php_phongo_decimal128_ce, 1, php_phongo_json_serializable_ce);
-	zend_class_implements(php_phongo_decimal128_ce, 1, php_phongo_type_ce);
-	zend_class_implements(php_phongo_decimal128_ce, 1, zend_ce_serializable);
 
 #if PHP_VERSION_ID >= 80000
 	zend_class_implements(php_phongo_decimal128_ce, 1, zend_ce_stringable);
@@ -383,4 +288,4 @@ void php_phongo_decimal128_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_decimal128.get_properties = php_phongo_decimal128_get_properties;
 	php_phongo_handler_decimal128.free_obj       = php_phongo_decimal128_free_object;
 	php_phongo_handler_decimal128.offset         = XtOffsetOf(php_phongo_decimal128_t, std);
-} /* }}} */
+}
