@@ -23,88 +23,59 @@
 #include "phongo_error.h"
 
 #include "MongoDB/WriteConcernError.h"
+#include "WriteConcernError_arginfo.h"
 
 zend_class_entry* php_phongo_writeconcernerror_ce;
 
-/* {{{ proto integer MongoDB\Driver\WriteConcernError::getCode()
-   Returns the MongoDB error code */
-static PHP_METHOD(WriteConcernError, getCode)
+PHONGO_DISABLED_CONSTRUCTOR(MongoDB_Driver_WriteConcernError)
+PHONGO_DISABLED_WAKEUP(MongoDB_Driver_WriteConcernError)
+
+/* Returns the MongoDB error code */
+static PHP_METHOD(MongoDB_Driver_WriteConcernError, getCode)
 {
-	zend_error_handling             error_handling;
 	php_phongo_writeconcernerror_t* intern;
 
 	intern = Z_WRITECONCERNERROR_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETURN_LONG(intern->code);
-} /* }}} */
+}
 
-/* {{{ proto object|null MongoDB\Driver\WriteConcernError::getInfo()
-   Returns additional metadata for the error */
-static PHP_METHOD(WriteConcernError, getInfo)
+/* Returns additional metadata for the error */
+static PHP_METHOD(MongoDB_Driver_WriteConcernError, getInfo)
 {
-	zend_error_handling             error_handling;
 	php_phongo_writeconcernerror_t* intern;
 
 	intern = Z_WRITECONCERNERROR_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	if (!Z_ISUNDEF(intern->info)) {
 		RETURN_ZVAL(&intern->info, 1, 0);
 	}
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\Driver\WriteConcernError::getMessage()
-   Returns the actual error message from the server */
-static PHP_METHOD(WriteConcernError, getMessage)
+/* Returns the actual error message from the server */
+static PHP_METHOD(MongoDB_Driver_WriteConcernError, getMessage)
 {
-	zend_error_handling             error_handling;
 	php_phongo_writeconcernerror_t* intern;
 
 	intern = Z_WRITECONCERNERROR_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	if (!intern->message) {
+		RETURN_STRING("");
 	}
-	zend_restore_error_handling(&error_handling);
 
 	RETURN_STRING(intern->message);
-} /* }}} */
+}
 
-/* {{{ MongoDB\Driver\WriteConcernError function entries */
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcernError_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_writeconcernerror_me[] = {
-	/* clang-format off */
-	PHP_ME(WriteConcernError, getCode, ai_WriteConcernError_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(WriteConcernError, getInfo, ai_WriteConcernError_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(WriteConcernError, getMessage, ai_WriteConcernError_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_WriteConcernError_void, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_WriteConcernError_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-	/* clang-format on */
-};
-/* }}} */
-
-/* {{{ MongoDB\Driver\WriteConcernError object handlers */
+/* MongoDB\Driver\WriteConcernError object handlers */
 static zend_object_handlers php_phongo_handler_writeconcernerror;
 
-static void php_phongo_writeconcernerror_free_object(zend_object* object) /* {{{ */
+static void php_phongo_writeconcernerror_free_object(zend_object* object)
 {
 	php_phongo_writeconcernerror_t* intern = Z_OBJ_WRITECONCERNERROR(object);
 
@@ -117,9 +88,9 @@ static void php_phongo_writeconcernerror_free_object(zend_object* object) /* {{{
 	if (!Z_ISUNDEF(intern->info)) {
 		zval_ptr_dtor(&intern->info);
 	}
-} /* }}} */
+}
 
-static zend_object* php_phongo_writeconcernerror_create_object(zend_class_entry* class_type) /* {{{ */
+static zend_object* php_phongo_writeconcernerror_create_object(zend_class_entry* class_type)
 {
 	php_phongo_writeconcernerror_t* intern = zend_object_alloc(sizeof(php_phongo_writeconcernerror_t), class_type);
 
@@ -129,9 +100,9 @@ static zend_object* php_phongo_writeconcernerror_create_object(zend_class_entry*
 	intern->std.handlers = &php_phongo_handler_writeconcernerror;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static HashTable* php_phongo_writeconcernerror_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_writeconcernerror_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	php_phongo_writeconcernerror_t* intern;
 	zval                            retval = ZVAL_STATIC_INIT;
@@ -140,7 +111,7 @@ static HashTable* php_phongo_writeconcernerror_get_debug_info(phongo_compat_obje
 	intern   = Z_OBJ_WRITECONCERNERROR(PHONGO_COMPAT_GET_OBJ(object));
 
 	array_init_size(&retval, 3);
-	ADD_ASSOC_STRING(&retval, "message", intern->message);
+	ADD_ASSOC_STRING(&retval, "message", intern->message ? intern->message : "");
 	ADD_ASSOC_LONG_EX(&retval, "code", intern->code);
 	if (!Z_ISUNDEF(intern->info)) {
 		Z_ADDREF(intern->info);
@@ -150,26 +121,21 @@ static HashTable* php_phongo_writeconcernerror_get_debug_info(phongo_compat_obje
 	}
 
 	return Z_ARRVAL(retval);
-} /* }}} */
-/* }}} */
+}
 
-void php_phongo_writeconcernerror_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_writeconcernerror_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "WriteConcernError", php_phongo_writeconcernerror_me);
-	php_phongo_writeconcernerror_ce                = zend_register_internal_class(&ce);
+	php_phongo_writeconcernerror_ce                = register_class_MongoDB_Driver_WriteConcernError();
 	php_phongo_writeconcernerror_ce->create_object = php_phongo_writeconcernerror_create_object;
-	PHONGO_CE_FINAL(php_phongo_writeconcernerror_ce);
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_writeconcernerror_ce);
 
 	memcpy(&php_phongo_handler_writeconcernerror, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_writeconcernerror.get_debug_info = php_phongo_writeconcernerror_get_debug_info;
 	php_phongo_handler_writeconcernerror.free_obj       = php_phongo_writeconcernerror_free_object;
 	php_phongo_handler_writeconcernerror.offset         = XtOffsetOf(php_phongo_writeconcernerror_t, std);
-} /* }}} */
+}
 
-zend_bool phongo_writeconcernerror_init(zval* return_value, bson_t* bson) /* {{{ */
+zend_bool phongo_writeconcernerror_init(zval* return_value, bson_t* bson)
 {
 	bson_iter_t                     iter;
 	php_phongo_writeconcernerror_t* intern;
@@ -196,7 +162,7 @@ zend_bool phongo_writeconcernerror_init(zval* return_value, bson_t* bson) /* {{{
 
 		bson_iter_document(&iter, &len, &data);
 
-		if (!php_phongo_bson_to_zval(data, len, &intern->info)) {
+		if (!php_phongo_bson_data_to_zval(data, len, &intern->info)) {
 			zval_ptr_dtor(&intern->info);
 			ZVAL_UNDEF(&intern->info);
 
@@ -205,4 +171,4 @@ zend_bool phongo_writeconcernerror_init(zval* return_value, bson_t* bson) /* {{{
 	}
 
 	return true;
-} /* }}} */
+}
