@@ -193,6 +193,8 @@ _mongoc_scram_destroy (mongoc_scram_t *scram)
    if (scram->cache) {
       _mongoc_scram_cache_destroy (scram->cache);
    }
+
+   memset (scram, 0, sizeof *scram);
 }
 
 
@@ -289,10 +291,10 @@ _mongoc_scram_start (mongoc_scram_t *scram,
       goto FAIL;
    }
 
-   scram->encoded_nonce_len = bson_b64_ntop (nonce,
-                                             sizeof (nonce),
-                                             scram->encoded_nonce,
-                                             sizeof (scram->encoded_nonce));
+   scram->encoded_nonce_len = mcommon_b64_ntop (nonce,
+                                                sizeof (nonce),
+                                                scram->encoded_nonce,
+                                                sizeof (scram->encoded_nonce));
 
    if (-1 == scram->encoded_nonce_len) {
       bson_set_error (error,
@@ -486,10 +488,10 @@ _mongoc_scram_generate_client_proof (mongoc_scram_t *scram,
       client_proof[i] = scram->client_key[i] ^ client_signature[i];
    }
 
-   r = bson_b64_ntop (client_proof,
-                      _scram_hash_size (scram),
-                      (char *) outbuf + *outbuflen,
-                      outbufmax - *outbuflen);
+   r = mcommon_b64_ntop (client_proof,
+                         _scram_hash_size (scram),
+                         (char *) outbuf + *outbuflen,
+                         outbufmax - *outbuflen);
 
    if (-1 == r) {
       return false;
@@ -701,7 +703,7 @@ _mongoc_scram_step2 (mongoc_scram_t *scram,
    }
 
    decoded_salt_len =
-      bson_b64_pton ((char *) val_s, decoded_salt, sizeof (decoded_salt));
+      mcommon_b64_pton ((char *) val_s, decoded_salt, sizeof (decoded_salt));
 
    if (-1 == decoded_salt_len) {
       bson_set_error (error,
@@ -836,10 +838,10 @@ _mongoc_scram_verify_server_signature (mongoc_scram_t *scram,
                        server_signature);
 
    encoded_server_signature_len =
-      bson_b64_ntop (server_signature,
-                     _scram_hash_size (scram),
-                     encoded_server_signature,
-                     sizeof (encoded_server_signature));
+      mcommon_b64_ntop (server_signature,
+                        _scram_hash_size (scram),
+                        encoded_server_signature,
+                        sizeof (encoded_server_signature));
    if (encoded_server_signature_len == -1) {
       return false;
    }
@@ -1130,6 +1132,8 @@ _mongoc_sasl_prep_impl (const char *name,
 char *
 _mongoc_sasl_prep (const char *in_utf8, int in_utf8_len, bson_error_t *err)
 {
+   BSON_UNUSED (in_utf8_len);
+
 #ifdef MONGOC_ENABLE_ICU
    return _mongoc_sasl_prep_impl ("password", in_utf8, in_utf8_len, err);
 #else
